@@ -2,13 +2,15 @@
 # this repository contains the full copyright notices and license terms.
 
 from trytond.pool import Pool
-from . import attendance
+
+from . import attendance, ir
 
 __all__ = ['register']
 
 
 def register():
     Pool.register(
+        ir.Rule,
         attendance.Line,
         attendance.Period,
         attendance.SheetLine,
