@@ -5,22 +5,18 @@ import datetime as dt
 from collections import defaultdict
 from itertools import chain
 
-try:
-    import pytz
-except ImportError:
-    pytz = None
-
-from sql import Window, Literal, Null, Column
+from sql import Column, Literal, Null, Window
 from sql.aggregate import Min, Sum
 from sql.conditionals import Coalesce
-from sql.functions import NthValue, CurrentTimestamp, Function
+from sql.functions import CurrentTimestamp, Function, NthValue
 
 from trytond import backend
 from trytond.cache import Cache
 from trytond.i18n import gettext
-from trytond.model import Workflow, ModelView, ModelSQL, fields
+from trytond.model import Index, ModelSQL, ModelView, Workflow, fields
 from trytond.pool import Pool, PoolMeta
 from trytond.pyson import Eval
+from trytond.tools import timezone as tz
 from trytond.transaction import Transaction
 
 from .exceptions import PeriodClosedError, PeriodTransitionError
@@ -42,14 +38,13 @@ class Line(ModelSQL, ModelView):
             ('company', '=', Eval('company')),
             ['OR',
                 ('start_date', '=', None),
-                ('start_date', '<=', Eval('date')),
+                ('start_date', '<=', Eval('date', None)),
                 ],
             ['OR',
                 ('end_date', '=', None),
-                ('end_date', '>=', Eval('date')),
+                ('end_date', '>=', Eval('date', None)),
                 ],
-            ],
-        depends=['company', 'date'])
+            ])
     at = fields.DateTime("At", required=True)
     date = fields.Date("Date", required=True)
     type = fields.Selection([
@@ -81,9 +76,9 @@ class Line(ModelSQL, ModelView):
         if not self.at:
             return
         at = self.at
-        if pytz and self.company and self.company.timezone:
-            timezone = pytz.timezone(self.company.timezone)
-            at = pytz.utc.localize(self.at, is_dst=None).astimezone(timezone)
+        if self.company and self.company.timezone:
+            timezone = tz.ZoneInfo(self.company.timezone)
+            at = self.at.replace(tzinfo=tz.UTC) .astimezone(timezone)
         return at.date()
 
     def get_rec_name(self, name):
@@ -124,14 +119,16 @@ class Line(ModelSQL, ModelView):
         super().delete(records)
 
     @classmethod
-    def validate(cls, records):
-        super().validate(records)
-        cls.check_closed_period(records)
+    def validate_fields(cls, records, field_names):
+        super().validate_fields(records, field_names)
+        cls.check_closed_period(records, field_names=field_names)
 
     @classmethod
-    def check_closed_period(cls, records, msg='modify'):
+    def check_closed_period(cls, records, msg='modify', field_names=None):
         pool = Pool()
         Period = pool.get('attendance.period')
+        if field_names and not (field_names & {'company', 'at'}):
+            return
         for record in records:
             period_date = Period.get_last_period_date(record.company)
             if period_date and period_date > record.at:
@@ -162,24 +159,28 @@ class Period(Workflow, ModelSQL, ModelView):
     _states = {
         'readonly': Eval('state') == 'closed',
         }
-    _depends = ['state']
 
     _last_period_cache = Cache('attendance.period', context=False)
 
-    ends_at = fields.DateTime("Ends at", required=True, states=_states,
-        depends=_depends)
+    ends_at = fields.DateTime("Ends at", required=True, states=_states)
     company = fields.Many2One('company.company', "Company", required=True,
-        states=_states, depends=_depends,
-        help="The company the period is associated with.")
+        states=_states, help="The company the period is associated with.")
     state = fields.Selection([
         ('draft', 'Draft'),
         ('closed', 'Closed'),
-        ], 'State', select=True, readonly=True,
+        ], "State", readonly=True, sort=False,
         help="The current state of the attendance period.")
 
     @classmethod
     def __setup__(cls):
         super().__setup__()
+        t = cls.__table__()
+        cls._sql_indexes.add(
+            Index(
+                t,
+                (t.company, Index.Equality()),
+                (t.state, Index.Equality()),
+                (t.ends_at, Index.Range(order='DESC'))))
         cls._transitions |= set((
                 ('draft', 'closed'),
                 ('closed', 'draft'),
